package IceBearSoft::PrintList;

# Print the list which may contain references to arrays, hashes and objects
# (C) Z. Wagner -- Ice Bear Soft, 17 Aug 2001
# ------------------------------------------------------------------------------
#  2 Aug 2003 -- print_list now prints the contents of objects

require Exporter;
@ISA = qw(Exporter);

sub new($*);

# Create a new object, no arguments

sub new {
my $class = shift;
my $this = {};
$this->{'indent'} = '';
return bless $this, $class;
}

# Print the list, inner arrays and hashes are indented.
# The first argument must be a scalar or a reference to something to print.
# The second optional argument prints a title followed by a colon and two spaces.
# Printing continues at the same line.

sub print_list {
  my ($this, $argument, $title) = @_;
  print "$title:  " if $title;
  $this->{'listed'} = {};
  $this->_list_($argument);
  delete $this->{'listed'};
}

# Internal function

sub _list_ {
  my ($this, $argument) = @_;
  return unless exists $this->{'listed'};
  my $listed = $this->{'listed'};
  my $rf = ref($argument);
  if (!$rf) {
    print $this->{'indent'}, "$argument\n";
  }
  elsif ($rf eq 'SCALAR') {
    $this->_list_($$argument);
  }
  elsif ($rf eq 'ARRAY') {
    print $this->{'indent'}, "$argument\n";
    $this->{'indent'} .= '  ';
    if (exists $listed->{$argument}) {
      print $this->{'indent'}, "... already listed.\n";
    } else {
      $listed->{$argument} = 1;
      foreach $item (@$argument) {
        $this->_list_($item);
      }
    }
    $this->{'indent'} = substr($this->{'indent'}, 2);
  }
  elsif ($rf eq 'GLOB') {
    print $this->{'indent'}, "$argument\n";
  }
  elsif ($rf eq 'CODE') {
    print $this->{'indent'}, "$argument\n";
  }
  else { # Hash or object
    print $this->{'indent'}, "$argument\n";
    $this->{'indent'} .= '  ';
    if (exists $listed->{$argument}) {
      print $this->{'indent'}, "... already listed.\n";
    } else {
      $listed->{$argument} = 1;
      foreach $key (sort keys %$argument) {
        print $this->{'indent'}, "$key: ";  $this->_list_($argument->{$key});
      }
    }
    $this->{'indent'} = substr($this->{'indent'}, 2);
  }
}
