package IceBearSoft::ZWsgml;

# Parsing SGML
# (C) Z. Wagner -- Ice Bear Soft, 3 Dec 1998
# 21 Jan 2005 -- attributes may also be within apostrophes
# 5 Mar 2005 -- the 'linenum' member contains the line number

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(attributes);

sub attributes($);
sub new($*);

# Accepts the class name and the reference to the open file

sub new {
  my ($class, $fh, @garbage) = @_;
  my $this = {};
  $this->{'file'} = $fh;
  $this->{'buffer'} = '';
  $this->{'linenum'} = 0;
  return bless $this, $class;
}

# reads the line into the buffer if necessary, should not be used

sub nextline {
  my $this = shift;
  my $fh = $this->{'file'};
  while (!defined($this->{'buffer'}) || length($this->{'buffer'})==0) {
    $this->{'buffer'} = <$fh> or last;
    $this->{'buffer'} =~ tr/\t/ /;
    $this->{'buffer'} =~ s/^\s*(.*)\s*$/$1/;
    $this->{'linenum'} = $this->{'linenum'} + 1;
  };
}

# returns array (key, value), if both are NULL, the file ends

sub nextTag {
  my $this = shift;
  my $key = '';
  my $value = '';
  $this->nextline if !$this->{'buffer'} || length($this->{'buffer'}) == 0;
  return ($key, $value) if !defined($this->{'buffer'}) || length($this->{'buffer'}) == 0;
  if ($this->{'buffer'} =~ /^</) {
    if ($this->{'buffer'} =~ /^<!--/) { # this is a comment, we will ignore it
      my $nesting = 1;
      my ($opening, $closing, $flushto);
      while ($nesting > 0) {
        $this->nextline if !$this->{'buffer'} || length($this->{'buffer'}) == 0;
        return ($key, $value) if !defined($this->{'buffer'}) || length($this->{'buffer'}) == 0;
        $opening = index($this->{'buffer'}, '<!--', 1);
        $closing = index($this->{'buffer'}, '-->');
        $flushto = -1;
        if ($opening < 0) {
          if ($closing < 0) { $this->{'buffer'} = ''; }
          else {
            $flushto = $closing + 2;  $nesting--;
          }
        }
        else {
          $flushto = $closing + 2;
          if ($closing < 0) {
            $flushto = $opening + 3;  $nesting++;
          }
          elsif ($opening > $closing) { $nesting--; }
        }
        if (length($this->{'buffer'}) > 0) {
          $this->{'buffer'} = substr($this->{'buffer'}, $flushto+1);
        }
      }
      $key = '!--';  $value = 'comment';
    }
    else { # tags cannot be nested
      my $T = $this->{'buffer'};  $this->{'buffer'} = '';
      while (index($T, '>') < 0) {
        $this->nextline;
        if (!defined($this->{'buffer'}) || length($this->{'buffer'})==0) {
          $this->{'buffer'} = '>';
        }
        $T .= ' ';  $T .= $this->{'buffer'};  $this->{'buffer'} = '';
      }
      $T =~ /^<(.*?)>(.*)$/;
      $T = $1 if defined($1);
      $this->{'buffer'} = $2 if defined($2);
      if ($T =~ /^(\S+)\s*(.*)\s*$/) {
        $key = lc($1) if defined($1);
        $value = $2 if defined($2);
      }
      else {
        $key = lc($T);
      }
    }
  }
  else { # normal text, look for tag within the line
    if ($this->{'buffer'} =~ /^(.*?)<(.*)$/) {
      $value = $1;  $this->{'buffer'} = '<';  $this->{'buffer'} .= $2;
    }
    else {
      $value = $this->{'buffer'};  $this->{'buffer'} = '';
    }
  }
  return ($key, $value, $this->{'buffer'});
}

# gets value from ZWsgml->nextTag, returns hash containing attribute's name => value

sub attributes {
  my $attr = shift;
  my %Attr = {};
  while (defined($attr) && length($attr) > 0) {
    if ($attr =~ /^(\S+?)\s*=\s*(.*)$/) { # key=value
      my $key = lc($1);
      my $rest = $2;
      if ($rest =~ /^('|")(.*?)\1\s*(.*)$/) { # "value"
        $rest = $2;  $attr = $3;
      }
      elsif ($rest =~ /^(\S+)\s*(.*)$/) { # value without quotes
        $rest = $1;  $attr = $2;
      }
      else { # empty value, possibly error
        print "\nEmpty value for attribute ", $key, "\n";
        $attr = $rest;  $rest = '';
      }
      $Attr{$key} = $rest;
    }
    elsif ($attr =~ /^(\S+)\s*(.*)$/) { # key without value
      $Attr{lc $1} = '';  $attr = $2;
    }
    else { # last key
      $Attr{lc $attr} = '';  $attr = '';
    }
  }
  return %Attr;
}
