package IceBearSoft::ZWurl;

# World Wide Web URL function(s)
# (C) Z. Wagner -- Ice Bear Soft, 27 Apr 1998
# Last modification: 21 Jan 2001

require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(
  abs_url
  split_url
  merge_url
  url_decode
  url_encode
  localize_url
  query_string
  cgi_parse
  parse_cookies
  http_url
  WeekDay
  Month
  DateTime
  LongDateTime
  set_cookie
  print_ref
);

sub abs_url($$);
sub split_url($);
sub url_decode($);
sub url_encode($);
sub localize_url($$);
sub query_string();
sub cgi_parse(;$);
sub parse_cookies(;$);
sub http_url($);
sub WeekDay($);
sub Month($);
sub DateTime($);
sub LongDateTime($);
sub set_cookie($);
sub print_ref($);

# Accepts referer (abs path) and possibly relative path. Returns abs path.
# Returns undefined value if the referer is invalid.

sub abs_url {
  my ($referer, $rel) = @_;
  my $scheme = 'http';
  my $host;
  my $port = '';
  my $object = '/';
  if ($referer =~ m!^([a-zA-Z]+)://([^:/\*]+)(:\d+)?(.*)$!) {
    $scheme = lc($1) if $1;
    $host = lc($2) if $2;
    $port = $3 if $3;
    $object = $4 if $4;
  }
  else {
    return $rel;
  }
  return $referer.$rel if $rel =~ /^#/;
  return $rel if $rel  =~ m!^[a-zA-Z]+:!;
  return $scheme . ':' . $rel if $rel  =~ m!^//([^:/\*]+)(:\d+)?(.*)$!;
  return $scheme . '://' . $host . $port . $rel
         if $rel eq '/' || $rel  =~ m!^(\*|/[^/].*)$!;
  if ($object =~ m!^(.*/)(.*)$!) {
    $object = $1 . $rel;
    while ($object =~ s![^/]+/\.\./!!) {}
    while ($object =~ s!/\./!/!) {}
    return $scheme . '://' . $host . $port . $object;
  }
  while ($rel =~ s![^/]+/\.\./!!) {}
  return $scheme . '://' . $host . $port . $rel;
}

# Accepts absolute URL and returns a hash containing 'url', 'scheme', 'host', 'port',
# 'object' and 'label' or 'search' (if present in the URL). The function also adds
# terminating slash if the URL ends with '.' or '..'

sub split_url {
  my $url = shift;
  my %hash;
  $hash{'url'} = $url;
  if ($url =~ m!^([a-zA-Z]+)://([^:/#\?\*]+)(:\d+)?([^#?]*)(\?[^#]*)?(#.*)?$!) {
    $hash{'scheme'} = lc($1) if $1;
    $hash{'host'} = lc($2) if $2;
    my $port = $3 if $3;
    if ($port) {
      $port =~ s/://;  $hash{'port'} = $port;
    }
    $hash{'object'} = $4 ? $4 : '/';
    $hash{'search'} = substr($5, 1) if $5;
    $hash{'label'} = substr($6, 1) if $6;
    if ($hash{'object'} =~ /\/\.{1,2}$/) {
      $hash{'object'} .= '/';
    }
  }
  elsif ($url =~ /^([a-zA-Z]+):(.*)$/) {
    $hash{'scheme'} = lc($1) if $1;
    $hash{'object'} = $2 if $2;
  }
  else {
    $hash{'scheme'} = "error: $url";
  }
  return %hash;
}

# Accepts a hash (in a form of split_url) and merges it to URL

sub merge_url (%) {
my %hash = @_;
if ($hash{'port'} eq '80') {
  delete $hash{'port'};
}
my $host = $hash{'host'};
$host .= ':'.$hash{'port'} if $hash{'port'};
my $uri = $hash{'object'};
$uri .= '?'.$hash{'search'} if $hash{'search'};
$uri .= '#'.$hash{'label'} if $hash{'label'};
return "$hash{'scheme'}://$host$uri";
}

# Accepts an URL encoded string and decodes it

sub url_decode {
  $_ = shift;
  s/\+/ /g;  s/%([0-9a-fA-F]{2}?)/chr hex($1)/ge;
  return $_;
}

# Accepts a string and URL-encodes it

sub url_encode {
  $_ = shift;
  s/([;%+&=\?\000-\037\177-\377#])/sprintf '%%%02x', ord($1)/ge;  s/ /+/g;
  return $_;
}

# Accepts referer (must be abs path) and any url, returns localized url if possible
# This does not normalize the URL if it refers to another host.
# Returns undefined value if the referer is invalid.

sub localize_url {
  my ($referer, $url) = @_;
  return $url
   if $url !~ /^[a-zA-Z]*:/ && $url !~ m!//! && $url !~ m!^/!;
  my %hash = split_url $referer;
  my $scheme = $hash{'scheme'};
  my $host = $hash{'host'};
  $hash{'object'} =~ m!^(.*)/[^/]*$!;
  my $path = $1 . '/';
  my $absurl = abs_url($referer, $url);
  return unless $absurl;
  %hash = split_url($absurl);
  return $url if $scheme ne $hash{'scheme'} or $host ne $hash{'host'};
  my $object = $hash{'object'};
  my $back = '';
  while (length($path) > 0 && $object !~ /^$path/) {
    $back .= '../';  $path =~ s!^(.*)/[^/]+/$!$1/!;
  }
  $object =~ s/^$path/$back/ if $path ne '';
  $object = $back . $object if $path eq '';
  $object = './' if $object eq '';
  $object .= '#'.$hash{'label'} if exists $hash{'label'};
  $object .= '?'.$hash{'search'} if $hash{'search'};
  return $object;
}

# Obtain the query string

sub query_string {
  my $query = '';
  if ($ENV{'REQUEST_METHOD'} eq 'GET') {
    $query = $ENV{'QUERY_STRING'};
  }
  else {
    $query = '';
    if ($ENV{'CONTENT_TYPE'} eq 'application/x-www-form-urlencoded') {
      while (<STDIN>) {
        $query .= $_;
      }
    }
  }
  return $query;
}

# Accepts a string with a CGI query and returns a hash with URL decoded key/values.
# If the argument is not specified, the function calls query_string

sub cgi_parse {
  my %hash;
  my $q = scalar(@_) == 0 ? query_string : shift;
  $q .= '&' if $q =~ /[^&]$/;
  while (length($q) > 0) {
    $q =~ /^([^&]*)&(.*)$/;
    my $pair = $1;  $q = $2;
    if ($pair =~ /^([^=]*)=(.*)$/) {
      $key = url_decode $1;
      $value = url_decode $2;
      if (exists($hash{$key})) {
        if (ref($hash{$key})) {
          push(@{$hash{$key}}, $value);
        }
        else {
          my @arr = ($hash{$key}, $value);
          $hash{$key} = \@arr
        }
      }
      else {
        $hash{$key} = $value;
      }
    }
  }
  return %hash;
}

# Accepts a string with cookies and returns them in a hash (URL decoded).
# If the argument is not specified, $ENV{'HTTP_COOKIE'} is used.

sub parse_cookies {
  my $q = scalar(@_) == 0 ? $ENV{'HTTP_COOKIE'} : shift;
  my %cookies;
  $q .= ';' if $q !~ /;\s*$/;
  while (length($q) > 0) {
    $q =~ /^\s*([^;]*);(.*)\s*$/;
    my $pair = $1;  $q = $2;
    if ($pair =~ /^([^=]*)=(.*)$/) {
      $cookies{url_decode $1} = url_decode $2;
    }
  }
  return %cookies;
}

# Accepts incomplete URL and prepends optional http:// (REXX does not allow // in parameters)

sub http_url {
  my $x = shift;
  return "http://$x" unless $x =~ /\/\//;
  return "http:$x" unless $x =~ /^(http|ftp|gopher|mailto|file|telnet):/i;
  return $x;
}

# Return the weekday name

sub WeekDay {
  # Array of week days
  my @weekday = ('Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday');
  return $weekday[shift];
}

# Return the month abbreviation

sub Month {
  # Abbreviation of months
  my @month = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
               'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
  return $month[shift];
}

# Make the date/time string as for use with Set-Cookie

sub LongDateTime {
  my $tm = shift;
  my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime($tm);
  $wday = WeekDay $wday;  $mon = Month $mon;
  $sec = "0$sec" if $sec < 10;
  $min = "0$min" if $min < 10;
  $hour = "0$hour" if $hour < 10;
  $mday = "0$mday" if $mday < 10;
  $year = "0$year" if $year < 10;
  return "$wday, $mday-$mon-$year $hour:$min:$sec GMT";
}

# The same but without the weekday, useful for file dates

sub DateTime {
  my $tm = shift;
  my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime($tm);
  $sec = "0$sec" if $sec < 10;
  $min = "0$min" if $min < 10;
  $hour = "0$hour" if $hour < 10;
  $mday = "0$mday" if $mday < 10;
  $year = "0$year" if $year < 10;
  return "$mday-$mon-$year $hour:$min:$sec GMT";
}

# Function for setting a cookie (you must use it yourself in Set-Cookie)
# Accepts a pointer to a hash with following variables:
# 'Name' = name of the cookie, required
# 'Value' = value of the cookie, required
# 'Path' = path for the cookie, optional
# 'Expires' = expiration (absolute value), optional

sub set_cookie {
  my $cookie = shift;
  return '' if !$cookie->{'Name'} || !$cookie->{'Value'};
  $Cookie = (url_encode $cookie->{'Name'}).'='.(url_encode $cookie->{'Value'});
  $Cookie .= "; path=$cookie->{'Path'}" if $cookie->{'Path'};
  if ($cookie->{'Expires'}) {
    my $expires = LongDateTime $cookie->{'Expires'};
    $Cookie .= "; expires=$expires";
  }
  return $Cookie;
}

# This function accepts a single variable which may be a reference to an
# array or to a hash and prints it as in HTML

sub print_ref {
  my $ref = shift;
  if (!ref($ref)) {
    print "$ref\n";
  }
  elsif (ref($ref) eq 'SCALAR') {
    print_ref $ref;
  }
  elsif (ref($ref) eq 'HASH') {
    print "<UL compact>\n";
    foreach $key (sort keys %$ref) {
      print "<LI><B>$key:</B> ";  print_ref $ref->{$key};
    }
    print "</UL>\n";
  }
  elsif (ref($ref) eq 'ARRAY') {
    print "<OL compact>\n";
    foreach $item (@$ref) {
      print '<LI>';  print_ref $item;
    }
    print "</OL>\n";
  }
  else {
    my $r = ref($ref);
    print "$r\n";
  }
}

__END__

We will have comments here...
