# This package defines a single function which gets either a package
# name or an object reference and returns a top-down list of all
# package names ending usually with 'Exporter'. This can be used if you
# wish to find whether the object is derived from a particular class.
# Suppose that you wish to check whether your object $obj is a
# "MyPackage" or derived from it by reblessing. You can use:
#
# die 'Invalid object type' unless scalar(grep(/^MyPackage$/, isA $obj));
#
# You can also supply the regular expression of valid types as the
# second optional parameter, e.g. isA $obj, '^My(First|Second)Package$'
# This will check whether the object is MyFirstPackage or
# MySecondPackage. In list context the function will return all
# matches, in scalar context it returns number of matches. You can thus
# write:
#
# die 'Invalid object type' unless scalar(isA $obj, '^My(First|Second)Package$')
#
# or just simply
#
# die 'Invalid object type' unless isA $obj, '^My(First|Second)Package$'

package IceBearSoft::isA;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(isA);

sub isA ($;$) {
  my ($o, $t) = @_;
  return isA(ref $o, $t) if ref $o;
  my @isA;
  push @isA, $o;
  foreach (@{$o . '::ISA'}) {
    push @isA, isA($_);
  }
  return $t ? grep(/$t/, @isA) : @isA;
}
