#!perl5
# HTTP retrieving
# Copyright (C) Z. Wagner -- Ice Bear Soft, 1 Apr 1999
# Last update: 10 Sep 2001
#
# Usage: http-retrieve.pl [options]
#
# List of options:
#       --url           absolute url
#       --file          file name to save as
#       --retry         retry count (default=50)
#       --proxy         proxy to be used, optional

use Getopt::Long;
use Carp;
use IceBearSoft::ZWdebug(list_hash);
use IceBearSoft::ZWurl(http_url);
use IceBearSoft::Zwebfun;

$result = GetOptions 'url=s', 'file=s', 'retry=i', 'proxy=s';
if (!$result || !$opt_file || !$opt_url) {
print <<'EOF' ;

Usage: http-retrieve.pl [options]

List of options:
       --url           absolute url
       --file          file name to save as
       --retry         retry count (default=50)
       --proxy         proxy to be used, optional
EOF
exit;
}
$opt_retry = 50 if (!$opt_retry || $opt_retry < 5);
$request{'url'} = http_url $opt_url;
$request{'file'} = $file = $opt_file;
$request{'proxy'} = $opt_proxy if $opt_proxy;
$http = new IceBearSoft::Zwebfun;
$nreq = 0;
do {
  $request{'method'} = $nreq > 0 ? 'GET' : 'HEAD';
  $response = $http->http(\%request);
  list_hash 'response', $response;
  $status = $response->{'Status'};
  $length = $response->{'Content-Length'} if $nreq == 0;
  $nreq++ if $status =~ /^2/ || $nreq > 1;
  $opt_retry = 1 if $nreq && !$length;
  if (open(F, $file)) {
    binmode(F);
    seek(F, 0, 2);
    $flen = tell(F);
    close F;
    $hdr{'Range'} = "bytes=$flen-";
    $request{'headers'} = \%hdr;
  }
} until $status =~ /^[34]/ || $length == $flen || $nreq > $opt_retry;
print "Final status: $status\n";
exit $status;
