#!perl5
# HTTP request
# Copyright (C) Z. Wagner -- Ice Bear Soft, 17 Jul 1998
#
# Usage: http.pl [options]
#
# List of options:
#       --method        method, GET | POST | HEAD | OPTIONS | TRACE
#       --url           absolute url
#       --host          host (server) and optional :port, ignored if --url defined
#       --object        object (optional, default='/'), ignored if --url defined
#       --proxy         proxy to be used, optional
#       --file          file for storing the object, optional
#       --bytes         byte range, optional but --file must always be given
#       --agent         User-Agent
#       --data          string for the POST method
#       --version       HTTP version (default: 1.1)

use Getopt::Long;
use Carp;
use IceBearSoft::PrintList;
use IceBearSoft::ZWurl(http_url);
use IceBearSoft::Zwebfun;

$result = GetOptions 'method=s', 'url=s', 'host|server=s', 'object=s', 'proxy=s',
                     'file=s', 'bytes=s', 'agent=s', 'data=s', 'version=s',
                     'hdr=s', 'referer=s';
if (!$result || !$opt_method) {
print <<'EOF' ;

Usage: http.pl [options]

List of options:
     --method        method, GET | POST | HEAD | OPTIONS | TRACE
     --url           absolute url
     --host          host (server) and optional :port, ignored if --url defined
     --object        object (optional, default='/'), ignored if --url defined
     --proxy         proxy to be used, optional
     --file          file for storing the object, optional
     --bytes         range in bytes, optional
     --agent         User-Agent
     --data          string for the POST method
     --version       HTTP version (default: 1.1)
EOF
exit;
}
croak 'Method must be specified' if !$opt_method;
if (!$opt_url) {
  croak 'No URL specified' if !opt_host;
  $opt_object = '/' if !$opt_object;
  $opt_url = 'http://' . $opt_host . $opt_object;
  $opt_host = $opt_host; # to satisfy -w
}
my $lister = new IceBearSoft::PrintList;
$request{'method'} = uc($opt_method);
$request{'url'} = http_url $opt_url;
$request{'proxy'} = $opt_proxy if $opt_proxy;
$request{'file'} = $opt_file if $opt_file;
$request{'data'} = $opt_data if $opt_data;
$request{'version'} = $opt_version if $opt_version;
if ($opt_bytes) {
  push @hdr, "Range: bytes=$opt_bytes";
}
if ($opt_agent) {
  push @hdr, "User-Agent: $opt_agent";
}
push @hdr, "Connection: close";
if ($opt_hdr) {
  push @hdr, $opt_hdr;
}
if ($opt_referer) {
  $opt_referer = 'Referer: ' . http_url $opt_referer;
  push @hdr, $opt_referer;
}
if (scalar(@hdr) > 0) {
  $request{'headers'} = \@hdr;
}
print 'request:  ';  $lister->print_list(\%request);
$http = new IceBearSoft::Zwebfun;
$response = $http->http(\%request);
print 'response:  ';  $lister->print_list($response);
exit $response{'Status'};
