#!perl5

use IceBearSoft::ZWurl(cgi_parse, parse_cookies, query_string);

sub envlist(@);
sub hashlist(%);

# Prepare the list of CGI variables

@cgi = qw(
  SERVER_SOFTWARE
  SERVER_NAME
  GATEWAY_INTERFACE
  SERVER_PROTOCOL
  SERVER_PORT
  REQUEST_METHOD
  HTTP_ACCEPT
  HTTP_REFERER
  HTTP_USER_AGENT
  PATH_INFO
  PATH_TRANSLATED
  REQUEST_URI
  SCRIPT_NAME
  QUERY_STRING
  REMOTE_HOST
  REMOTE_ADDR
  REMOTE_PORT
  REMOTE_USER
  AUTH_TYPE
  CONTENT_TYPE
  CONTENT_LENGTH
  HTTP_COOKIES
);

# Apache extensions

@apache = qw(
  SERVER_ADMIN
  SERVER_SIGNATURE
  QUERY_STRING_UNESCAPED
  REDIRECT_QUERY_STRING
  REDIRECT_URL
  UNIQUE_ID
);
#  It may be dangerous to show the following settings to everybody
#  SCRIPT_FILENAME
#  DOCUMENT_ROOT

# Header of the output

print "Content-type: text/html\n\n";

print "<HTML>\n";
print "<HEAD>\n";
print "<TITLE>CGI/1.0 test script report:</TITLE>\n";
print "</HEAD>\n";
print "<BODY>\n";
print "<H1>CGI/1.0 test script report:</H1>\n";
print 'This CGI script was written in Perl 5 by <a href="mailto:wagner@mbox.cesnet.cz">';
print "Z. Wagner</a>.<p>\n";
# print $ENV{'CONTENT_LENGTH'}, "\n";
print "<H2>General information</H2>\n";
print "<UL>\n";
envlist @cgi;
print "</UL>\n";

# Apache extensions

print "<H2>Apache extensions</H2>\n";
print "<UL>\n";
envlist @apache;
print<<EOD;
<LI>SCRIPT_FILENAME <i>is intentionally hidden (full file name of the script)</i>
<LI>DOCUMENT_ROOT <i>is intentionally hidden</i>
EOD
print "</UL>\n";

# CGI values

$query = query_string;
print "<H2>Query</H2>\n<P><code>$query</code>\n";
if ($ENV{'CONTENT_TYPE'} =~ /^multipart\/form-data/) {
  print "multipart:<pre>\n";
  while (<STDIN>) {
    print $_;
  }
  print "</pre>\n"
}

# Parse the CGI values

%cgi = cgi_parse $query;
print "<H2>Parsed values</H2>\n";
print "<UL>\n";
hashlist %cgi;
print "</UL>\n";

if ($ENV{'HTTP_COOKIES'}) {
  %cookies = parse_cookies $ENV{'HTTP_COOKIES'};
  print "<H2>Parsed cookies</H2>\n";
  print "<UL>\n";
  hashlist %cookies;
  print "</UL>\n";
}
else {
  print "<H2>No cookies!</H2>";
}

print "</BODY>\n</HTML>\n";

# List the environment variables the names of which are given in the list

sub envlist {
  my @list = @_;
  foreach $name (@list) {
    print "<LI>$name = ", $ENV{$name}, "\n";
  }
}

# List the sorted hash

sub hashlist {
  my %hash = @_;
  foreach $key (sort keys %hash) {
    if (ref($hash{$key})) {
      print "<LI>$key:\n<UL compact type=square>\n";
      foreach $v (sort(@{$hash{$key}})) {
        print "<LI>$v\n";
      }
      print "</UL>\n";
    }
    else {
      print "<LI>$key = ", $hash{$key}, "\n";
    }
  }
}
