#!/usr/bin/perl

use Config;
use Getopt::Long;
use File::Path;

sub install ($$);

$result = GetOptions 'libdir=s', 'bindir=s', 'cgidir=s', 'htmldoc=s';
die <<HELP
Usage:
install.pl --libdir=<perl library directory> \\
           --bindir=<directory with binaries> \\
           --cgidir=<CGI scripts directory> \\
           --htmldoc=<directory of HTML documents containing webtools subdir>

Default library directory is taken first from \$ENV{'PERL5LIB'},
then from \$ENV{'PERLLIB'} if it does not take path separator.

Default binary directory is the first segment of \$ENV{'PATH'}
which starts with \$ENV{'ZWPERL'} (if it is defined).

Default CGI directory is taken from \$ENV{'CGIDIR'}.

Default HTML directory is taken from \$ENV{'HTMLDOC'}.
HELP
unless $result;

# Get the OS specific parameters
if ($Config{'osname'} =~ /^os\/2|windows/i) {
  $copy = 'copy';
  $pathsep = ';';
  $optre = '(?i)';
} else {
  $copy = 'cp -p';
  $pathsep = ':';
  $optre = '';
}

# set libdir
if (!$opt_libdir) {
  my $dir = $ENV{'PERL5LIB'};
  $opt_libdir = $dir if $dir && $dir !~ /$pathsep/;
}
if (!$opt_libdir) {
  my $dir = $ENV{'PERLLIB'};
  $opt_libdir = $dir if $dir && $dir !~ /$pathsep/;
}
$opt_libdir = '' if $opt_libdir =~ /$pathsep/;

# set cgidir
$opt_cgidir = $ENV{'CGIDIR'} unless $opt_cgidir;
$opt_cgidir = '' if $opt_cgidir =~ /$pathsep/;

# set htmldoc
$opt_htmldoc = $ENV{'HTMLDOC'} unless $opt_htmldoc;
$opt_htmldoc = '' if $opt_htmldoc =~ /$pathsep/;

# set bindir
if (!$opt_bindir) {
  my $dir = $ENV{'PATH'};
  $dir =~ s/\\/\//g;
  my @path = split /$pathsep/, $dir;
  $dir = $ENV{'ZWPERL'};
  $dir =~ s/\\/\//g;
  if ($dir && $dir !~ /$pathsep/) {
    foreach (@path) {
      if (/^$optre$dir/) {
        $opt_bindir = $_;
        last;
      }
    }
  }
}
$opt_bindir = '' if $opt_bindir =~ /$pathsep/;

die <<ERR
Wrong directory setting:
bindir = $opt_bindir
cgidir = $opt_cgidir
libdir = $opt_libdir
ERR
unless $opt_bindir && $opt_cgidir && $opt_libdir;

install 'IceBearSoft/*', "$opt_libdir/IceBearSoft";
install 'cgi/*', "$opt_cgidir";
install 'bin/*', "$opt_bindir";
install 'webtools/*', "$opt_htmldoc/webtools";

sub install {
  my ($src, $dest) = @_;
  mkpath $dest, 0, 0777 unless -d $dest;
  my $cmd = "$copy $src $dest";
  $cmd =~ s/\//\\/g if $optre;
  print `$cmd\n`;
}
